#!/bin/bash
#
# bash-common-api:json-api.sh
#
# function load_vars_from_json # DEPRECATED 2021-10-04, DISABLED 2023-03-07, REMOVED 2024-09-25

if [ "$( type -t echofatal )" != 'function' ] ; then
   # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/message-api.sh" || exit $?
fi

__BASH_COMMON_API_JSON_JQ_LIB__="$( realpath "$( dirname "${BASH_SOURCE[0]}" )/jq" )"
declare -grx __BASH_COMMON_API_JSON_JQ_LIB__

function __jq_load_vars_from_json_run_jq__ {
  jq -r -L "${__BASH_COMMON_API_JSON_JQ_LIB__}" \
     --arg MESSAGE_API_SHOW_DEBUG "${MESSAGE_API_SHOW_DEBUG:-}" \
     'include "load-vars-from-json"; load_vars_from_json_string | .[] | @sh' || return $?
}

function __jq_load_vars_from_json_string_exit_on_error__ { # Require by jq/load-vars-from-json.jq:load_vars_from_json_string
  local -ri rc="$?"

  if [ "${rc}" -ne 0 ] ; then
    exit "${rc}"
  fi
}

function __jq_load_vars_from_json_string_load_var__ { # Require by jq/load-vars-from-json.jq:load_vars_from_json_string
  echoinfo_quiet "Discover '${__JQ_K__}'"

  if [ "${__JQ_N__}" = 'true' ] ; then
    echoinfo_quiet "Value for '${__JQ_K__}' is null (ignore)"
    return 0
  fi

  local -a declare_opts=(-gx)
  if [ "${BASH_COMMON_API_JSON_API_LOAD_VAR_READWRITE:-}" != 'true' ] ; then
    declare_opts+=(-r)
  fi
  declare "${declare_opts[@]}" "${__JQ_K__}"="$( envsubst <<<"${__JQ_V__}" )" || true # (x) required by envsubst (next iteration)

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug "Set '${__JQ_K__}'='${!__JQ_K__}'"
  fi
}

function __load_var_from_json_string__ { # For compatibility mode only
  echoinfo_quiet "Discover '${key}'"

  local value_json=
  value_json="$( jq --arg key "${key}" '.[$key]' <<<"${json}" )" || return $?

  if [ "${value_json}" = 'null' ] ; then
    echoinfo_quiet "Value for '${key}' is null (ignore)"
    return 0
  fi

  local value=
  value="$( jq -r --arg key "${key}" '.[$key]' <<<"${json}" )" || return $?

  local -a declare_opts=(-gx)
  if [ "${BASH_COMMON_API_JSON_API_LOAD_VAR_READWRITE:-}" != 'true' ] ; then
    declare_opts+=(-r)
  fi
  declare "${declare_opts[@]}" "${key}"="$( envsubst <<<"${value}" )" || true # (x) required by envsubst (next iteration)

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug "Set '${key}'='${!key}'"
  fi
}

##
## get_expanded_value_from_json '{"A":"${HOME}"}' '.A'
## - Extract value from JSON string using jq filter and resolv bash variables
## - jq is run using --exit-status, meaning that : the exit status of jq to 0
##   if the last output value was neither false nor null, 1 if the last output
##   value was either false or null, or 4  if no valid result was ever produced.
##   Normally jq exits with 2 if there was any usage problem or system error,
##   3 if there was a jq program  compile  error, or 0 if the jq program ran
##
function get_expanded_value_from_json {
  local -r json_str="$1"
  local -r key="$2"

  local value=
  value="$( jq -rce "${key}" <<<"${json_str}" )" || return $?

  envsubst <<<"${value}" || return $?
}

##
## get_optional_expanded_value_from_json '{"A":"${HOME}"}' '.A'
## - Extract value from JSON string using jq filter and resolv bash variables
##
function get_optional_expanded_value_from_json {
  local -r json_str="$1"
  local -r key="$2"

  local value=
  value="$( jq -rc "${key}" <<<"${json_str}" )" || return $?

  envsubst <<<"${value}" || return $?
}

##
## is_null_or_empty 'null'
## - exit 0 is given string is 'null' ou '' (empty), exit 1 otherwise
##
function is_null_or_empty {
  local -r value="$1"

  if [ "${value}" = 'null' ] ; then
    return 0
  elif [ -z "${value}" ] ; then
    return 0
  fi

  return 1
}

##
## load_vars_from_json_file /path/to/file.json '.vars'
## - Initialize bash variables from JSON file
##
function load_vars_from_json_file {
  local -r json_file="$1"
  local -r json_filter="$2"

  if [ ! -f "${json_file}" ] ; then
    echofatal "${FUNCNAME[0]}() - Can't find JSON file: '${json_file}'."
    return 100
  fi

  local json=
  json="$( jq -r "${json_filter}" "${json_file}" )" || return $?
  local -r json

  if [ "${json}" = 'null' ] ; then
    echofatal "${FUNCNAME[0]}() - Can't find values for JSON filter: '${json_filter}'."
    return 100
  fi

  load_vars_from_json_string "${json}" || return $?
}

##
## load_vars_from_json_string '{"A":"a","B":"${HOME}"}' '.vars'
## - Initialize bash variables from JSON string
##
function load_vars_from_json_string {
  local -r json="$1"

  local keys=
  local key=

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    keys="$( jq -r 'keys[] | select( contains( "@" ) )' <<<"${json}" )" || return $?

    while read -r key ; do
      if [ -n "${key}" ] ; then
        echodebug "Ignore '${key}'"
      fi
    done <<<"${keys}"
  fi

  if [ "${BASH_COMMON_API_JSON_API_LOAD_VAR_COMPATIBILITY:-}" != 'true' ] ; then
    # shellcheck disable=SC1090
    source <( __jq_load_vars_from_json_run_jq__ <<<"${json}" || return $? )
  else
    # Typically ignore all keys like "VARNAME@DISABLED"
    keys="$( jq -r 'keys[] | select( contains( "@" ) | not )' <<<"${json}" )" || return $?
    local -r keys

    while read -r key ; do
      if [ -n "${key}" ] ; then
        __load_var_from_json_string__ || return $?
      fi
    done <<<"${keys}"
  fi
}

