#!/bin/bash
#
# bash-common-api:message-api.sh
#

#
# Handle colors
#
if [ "${MESSAGE_API_NOCOLOR:-}" != 'true' ] ; then
  declare -r BLUE=$'\033[0;34m'
  declare -r CYAN=$'\033[0;36m'
  declare -r GREEN=$'\033[0;32m'
  declare -r MAGENTA=$'\033[0;35m'
  declare -r RED=$'\033[0;31m'
  declare -r YELLOW=$'\033[0;33m'

  declare -r LIGHT_BLUE=$'\033[1;34m'
  declare -r LIGHT_CYAN=$'\033[1;36m'
  declare -r LIGHT_GREEN=$'\033[1;32m'
  declare -r LIGHT_MAGENTA=$'\033[1;35m'
  declare -r LIGHT_RED=$'\033[1;31m'
  declare -r LIGHT_YELLOW=$'\033[1;33m'

  declare -r NOCOLOR=$'\033[0m' # No Color
else
  declare -r BLUE=
  declare -r CYAN=
  declare -r GREEN=
  declare -r MAGENTA=
  declare -r RED=
  declare -r YELLOW=

  declare -r LIGHT_BLUE=
  declare -r LIGHT_CYAN=
  declare -r LIGHT_GREEN=
  declare -r LIGHT_MAGENTA=
  declare -r LIGHT_RED=
  declare -r LIGHT_YELLOW=

  declare -r NOCOLOR=
fi

# Just to avoid shellcheck warnings, be able to have a sample using:
# ( source message-api.sh ; test_colors)
function test_colors {
  echo "${BLUE}BLUE ${CYAN}CYAN ${GREEN}GREEN ${MAGENTA}MAGENTA ${RED}RED ${YELLOW}YELLOW${NOCOLOR}"
  echo "${LIGHT_BLUE}LIGHT_BLUE ${LIGHT_CYAN}LIGHT_CYAN ${LIGHT_GREEN}LIGHT_GREEN ${LIGHT_MAGENTA}LIGHT_MAGENTA ${LIGHT_RED}LIGHT_RED ${LIGHT_YELLOW}LIGHT_YELLOW${NOCOLOR}"

  echodebug 'echodebug'
  echoerror 'echoerror'
  echofatal 'echofatal'
  echoinfo 'echoinfo'
  echookay 'echookay'
  echowarn 'echowarn'
}

function __message_api_get_date__ {
  if [ "${MESSAGE_API_HIDE_DATE:-}" != 'true' ] ; then
    echo " $( date )"
  fi
}

# Use stderr (&2)
function echofatal {
  ( set +x
  cat <<EOF >&2
*** [${RED}FATAL${NOCOLOR}]$( __message_api_get_date__ ) - ${RED}$@${NOCOLOR}
EOF
  )
}

# Use stderr (&2)
function echoerror {
  ( set +x
  cat <<EOF >&2
*** [${LIGHT_RED}ERROR${NOCOLOR}]$( __message_api_get_date__ ) - ${LIGHT_RED}$@${NOCOLOR}
EOF
  )
}

function __echowarn__ {
  cat <<EOF
*** [${CYAN}WARN${NOCOLOR}]$( __message_api_get_date__ ) - ${CYAN}$@${NOCOLOR}
EOF
}

# Use stdout (&1) if MESSAGE_API_WARN_STDOUT = true
function echowarn {
  ( set +x
  if [ "${MESSAGE_API_WARN_STDOUT:-}" = 'true' ] ; then
    __echowarn__ "$@"
  else
    __echowarn__ "$@" >&2
  fi
  )
}

# Use stderr (&2)
# Theses messages are only display if: MESSAGE_API_SHOW_DEBUG='true'
function echodebug {
  ( set +x
  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    cat <<EOF >&2
*** [${LIGHT_MAGENTA}DEBUG${NOCOLOR}]$( __message_api_get_date__ ) - ${LIGHT_MAGENTA}$@${NOCOLOR}
EOF
  fi
  )
}

function __echoinfo__ {
  cat <<EOF
* [${LIGHT_YELLOW}INFO${NOCOLOR}]$( __message_api_get_date__ ) - ${LIGHT_YELLOW}$@${NOCOLOR}
EOF
}

# Use stdout (&1) if MESSAGE_API_INFO_STDOUT = true
function echoinfo {
  ( set +x
  if [ "${MESSAGE_API_INFO_STDOUT:-}" = 'true' ] ; then
    __echoinfo__ "$@"
  else
    __echoinfo__ "$@" >&2
  fi
  )
}

function __echookay__ {
  cat <<EOF
* [${LIGHT_GREEN}OK${NOCOLOR}]$( __message_api_get_date__ ) - ${LIGHT_GREEN}$@${NOCOLOR}
EOF
}

# Use stdout (&1) if MESSAGE_API_OKAY_STDOUT = true
function echookay {
  ( set +x
  if [ "${MESSAGE_API_OKAY_STDOUT:-}" = 'true' ] ; then
    __echookay__ "$@"
  else
    __echookay__ "$@" >&2
  fi
  )
}

# Use stdout (&1) if MESSAGE_API_INFO_STDOUT = true
# Theses messages are not display if: MESSAGE_API_QUIET='true'
function echoinfo_quiet {
  ( set +x
  if [ "${MESSAGE_API_QUIET:-}" != 'true' ] ; then
    echoinfo "$@"
  fi
  )
}

# Use stdout (&1) if MESSAGE_API_OKAY_STDOUT = true
# Theses messages are not display if: MESSAGE_API_QUIET='true'
function echookay_quiet {
  ( set +x
  if [ "${MESSAGE_API_QUIET:-}" != 'true' ] ; then
    echookay "$@"
  fi
  )
}
